@@ -7,7 +7,7 @@
 import { vi, describe, it, expect, beforeEach, afterEach } from 'vitest';
 import { listMcpServers } from './list.js';
 import { loadSettings } from '../../config/settings.js';
-import { loadExtensions } from '../../config/extension.js';
+import { ExtensionStorage, loadExtensions } from '../../config/extension.js';
 import { createTransport } from '@google/gemini-cli-core';
 import { Client } from '@modelcontextprotocol/sdk/client/index.js';
 
@@ -16,6 +16,9 @@ vi.mock('../../config/settings.js', () => ({
 }));
 vi.mock('../../config/extension.js', () => ({
   loadExtensions: vi.fn(),
+  ExtensionStorage: {
+    getUserExtensionsDir: vi.fn(),
+  },
 }));
 vi.mock('@google/gemini-cli-core', () => ({
   createTransport: vi.fn(),
@@ -34,6 +37,7 @@ vi.mock('@google/gemini-cli-core', () => ({
 }));
 vi.mock('@modelcontextprotocol/sdk/client/index.js');
 
+const mockedExtensionStorage = ExtensionStorage as vi.Mock;
 const mockedLoadSettings = loadSettings as vi.Mock;
 const mockedLoadExtensions = loadExtensions as vi.Mock;
 const mockedCreateTransport = createTransport as vi.Mock;
@@ -69,6 +73,9 @@ describe('mcp list command', () => {
     MockedClient.mockImplementation(() => mockClient);
     mockedCreateTransport.mockResolvedValue(mockTransport);
     mockedLoadExtensions.mockReturnValue([]);
+    mockedExtensionStorage.getUserExtensionsDir.mockReturnValue(
+      '/mocked/extensions/dir',
+    );
   });
 
   afterEach(() => {
