@@ -10,14 +10,17 @@ import * as os from 'node:os';
 import * as path from 'node:path';
 import {
   EXTENSIONS_CONFIG_FILENAME,
+  ExtensionStorage,
   INSTALL_METADATA_FILENAME,
   annotateActiveExtensions,
   disableExtension,
   enableExtension,
   installExtension,
   loadExtension,
+  loadExtensionConfig,
   loadExtensions,
   performWorkspaceExtensionMigration,
+  requestConsentNonInteractive,
   uninstallExtension,
   type Extension,
 } from './extension.js';
@@ -25,6 +28,7 @@ import {
   GEMINI_DIR,
   type GeminiCLIExtension,
   ExtensionUninstallEvent,
+  ExtensionDisableEvent,
   ExtensionEnableEvent,
 } from '@google/gemini-cli-core';
 import { execSync } from 'node:child_process';
@@ -71,6 +75,7 @@ vi.mock('./trustedFolders.js', async (importOriginal) => {
 const mockLogExtensionEnable = vi.hoisted(() => vi.fn());
 const mockLogExtensionInstallEvent = vi.hoisted(() => vi.fn());
 const mockLogExtensionUninstall = vi.hoisted(() => vi.fn());
+const mockLogExtensionDisable = vi.hoisted(() => vi.fn());
 vi.mock('@google/gemini-cli-core', async (importOriginal) => {
   const actual =
     await importOriginal<typeof import('@google/gemini-cli-core')>();
@@ -79,9 +84,11 @@ vi.mock('@google/gemini-cli-core', async (importOriginal) => {
     logExtensionEnable: mockLogExtensionEnable,
     logExtensionInstallEvent: mockLogExtensionInstallEvent,
     logExtensionUninstall: mockLogExtensionUninstall,
+    logExtensionDisable: mockLogExtensionDisable,
     ExtensionEnableEvent: vi.fn(),
     ExtensionInstallEvent: vi.fn(),
     ExtensionUninstallEvent: vi.fn(),
+    ExtensionDisableEvent: vi.fn(),
   };
 });
 
@@ -146,7 +153,9 @@ describe('extension tests', () => {
         version: '1.0.0',
       });
 
-      const extensions = loadExtensions();
+      const extensions = loadExtensions(
+        new ExtensionEnablementManager(ExtensionStorage.getUserExtensionsDir()),
+      );
       expect(extensions).toHaveLength(1);
       expect(extensions[0].path).toBe(extensionDir);
       expect(extensions[0].config.name).toBe('test-extension');
@@ -165,7 +174,9 @@ describe('extension tests', () => {
         version: '2.0.0',
       });
 
-      const extensions = loadExtensions();
+      const extensions = loadExtensions(
+        new ExtensionEnablementManager(ExtensionStorage.getUserExtensionsDir()),
+      );
 
       expect(extensions).toHaveLength(2);
       const ext1 = extensions.find((e) => e.config.name === 'ext1');
@@ -185,7 +196,9 @@ describe('extension tests', () => {
         contextFileName: 'my-context-file.md',
       });
 
-      const extensions = loadExtensions();
+      const extensions = loadExtensions(
+        new ExtensionEnablementManager(ExtensionStorage.getUserExtensionsDir()),
+      );
 
       expect(extensions).toHaveLength(1);
       const ext1 = extensions.find((e) => e.config.name === 'ext1');
@@ -210,11 +223,14 @@ describe('extension tests', () => {
         SettingScope.User,
         tempWorkspaceDir,
       );
-      const extensions = loadExtensions();
+      const manager = new ExtensionEnablementManager(
+        ExtensionStorage.getUserExtensionsDir(),
+      );
+      const extensions = loadExtensions(manager);
       const activeExtensions = annotateActiveExtensions(
         extensions,
-        [],
         tempWorkspaceDir,
+        manager,
       ).filter((e) => e.isActive);
       expect(activeExtensions).toHaveLength(1);
       expect(activeExtensions[0].name).toBe('enabled-extension');
@@ -234,7 +250,9 @@ describe('extension tests', () => {
         },
       });
 
-      const extensions = loadExtensions();
+      const extensions = loadExtensions(
+        new ExtensionEnablementManager(ExtensionStorage.getUserExtensionsDir()),
+      );
       expect(extensions).toHaveLength(1);
       const loadedConfig = extensions[0].config;
       const expectedCwd = path.join(
@@ -254,13 +272,18 @@ describe('extension tests', () => {
       });
       fs.writeFileSync(path.join(sourceExtDir, 'context.md'), 'linked context');
 
-      const extensionName = await installExtension({
-        source: sourceExtDir,
-        type: 'link',
-      });
+      const extensionName = await installExtension(
+        {
+          source: sourceExtDir,
+          type: 'link',
+        },
+        async (_) => true,
+      );
 
       expect(extensionName).toEqual('my-linked-extension');
-      const extensions = loadExtensions();
+      const extensions = loadExtensions(
+        new ExtensionEnablementManager(ExtensionStorage.getUserExtensionsDir()),
+      );
       expect(extensions).toHaveLength(1);
 
       const linkedExt = extensions[0];
@@ -309,7 +332,11 @@ describe('extension tests', () => {
         };
         fs.writeFileSync(configPath, JSON.stringify(extensionConfig));
 
-        const extensions = loadExtensions();
+        const extensions = loadExtensions(
+          new ExtensionEnablementManager(
+            ExtensionStorage.getUserExtensionsDir(),
+          ),
+        );
 
         expect(extensions).toHaveLength(1);
         const extension = extensions[0];
@@ -360,7 +387,9 @@ describe('extension tests', () => {
         JSON.stringify(extensionConfig),
       );
 
-      const extensions = loadExtensions();
+      const extensions = loadExtensions(
+        new ExtensionEnablementManager(ExtensionStorage.getUserExtensionsDir()),
+      );
 
       expect(extensions).toHaveLength(1);
       const extension = extensions[0];
@@ -388,7 +417,9 @@ describe('extension tests', () => {
       const badConfigPath = path.join(badExtDir, EXTENSIONS_CONFIG_FILENAME);
       fs.writeFileSync(badConfigPath, '{ "name": "bad-ext"'); // Malformed
 
-      const extensions = loadExtensions();
+      const extensions = loadExtensions(
+        new ExtensionEnablementManager(ExtensionStorage.getUserExtensionsDir()),
+      );
 
       expect(extensions).toHaveLength(1);
       expect(extensions[0].config.name).toBe('good-ext');
@@ -420,7 +451,9 @@ describe('extension tests', () => {
       const badConfigPath = path.join(badExtDir, EXTENSIONS_CONFIG_FILENAME);
       fs.writeFileSync(badConfigPath, JSON.stringify({ version: '1.0.0' }));
 
-      const extensions = loadExtensions();
+      const extensions = loadExtensions(
+        new ExtensionEnablementManager(ExtensionStorage.getUserExtensionsDir()),
+      );
 
       expect(extensions).toHaveLength(1);
       expect(extensions[0].config.name).toBe('good-ext');
@@ -448,11 +481,35 @@ describe('extension tests', () => {
         },
       });
 
-      const extensions = loadExtensions();
+      const extensions = loadExtensions(
+        new ExtensionEnablementManager(ExtensionStorage.getUserExtensionsDir()),
+      );
       expect(extensions).toHaveLength(1);
       const loadedConfig = extensions[0].config;
       expect(loadedConfig.mcpServers?.['test-server'].trust).toBeUndefined();
     });
+
+    it('should throw an error for invalid extension names', () => {
+      const consoleSpy = vi
+        .spyOn(console, 'error')
+        .mockImplementation(() => {});
+      const badExtDir = createExtension({
+        extensionsDir: userExtensionsDir,
+        name: 'bad_name',
+        version: '1.0.0',
+      });
+
+      const extension = loadExtension({
+        extensionDir: badExtDir,
+        workspaceDir: tempWorkspaceDir,
+      });
+
+      expect(extension).toBeNull();
+      expect(consoleSpy).toHaveBeenCalledWith(
+        expect.stringContaining('Invalid extension name: "bad_name"'),
+      );
+      consoleSpy.mockRestore();
+    });
   });
 
   describe('annotateActiveExtensions', () => {
@@ -477,8 +534,8 @@ describe('extension tests', () => {
     it('should mark all extensions as active if no enabled extensions are provided', () => {
       const activeExtensions = annotateActiveExtensions(
         extensions,
-        [],
         '/path/to/workspace',
+        new ExtensionEnablementManager(ExtensionStorage.getUserExtensionsDir()),
       );
       expect(activeExtensions).toHaveLength(3);
       expect(activeExtensions.every((e) => e.isActive)).toBe(true);
@@ -487,8 +544,11 @@ describe('extension tests', () => {
     it('should mark only the enabled extensions as active', () => {
       const activeExtensions = annotateActiveExtensions(
         extensions,
-        ['ext1', 'ext3'],
         '/path/to/workspace',
+        new ExtensionEnablementManager(
+          ExtensionStorage.getUserExtensionsDir(),
+          ['ext1', 'ext3'],
+        ),
       );
       expect(activeExtensions).toHaveLength(3);
       expect(activeExtensions.find((e) => e.name === 'ext1')?.isActive).toBe(
@@ -505,8 +565,11 @@ describe('extension tests', () => {
     it('should mark all extensions as inactive when "none" is provided', () => {
       const activeExtensions = annotateActiveExtensions(
         extensions,
-        ['none'],
         '/path/to/workspace',
+        new ExtensionEnablementManager(
+          ExtensionStorage.getUserExtensionsDir(),
+          ['none'],
+        ),
       );
       expect(activeExtensions).toHaveLength(3);
       expect(activeExtensions.every((e) => !e.isActive)).toBe(true);
@@ -515,8 +578,11 @@ describe('extension tests', () => {
     it('should handle case-insensitivity', () => {
       const activeExtensions = annotateActiveExtensions(
         extensions,
-        ['EXT1'],
         '/path/to/workspace',
+        new ExtensionEnablementManager(
+          ExtensionStorage.getUserExtensionsDir(),
+          ['EXT1'],
+        ),
       );
       expect(activeExtensions.find((e) => e.name === 'ext1')?.isActive).toBe(
         true,
@@ -527,7 +593,14 @@ describe('extension tests', () => {
       const consoleSpy = vi
         .spyOn(console, 'error')
         .mockImplementation(() => {});
-      annotateActiveExtensions(extensions, ['ext4'], '/path/to/workspace');
+      annotateActiveExtensions(
+        extensions,
+        '/path/to/workspace',
+        new ExtensionEnablementManager(
+          ExtensionStorage.getUserExtensionsDir(),
+          ['ext4'],
+        ),
+      );
       expect(consoleSpy).toHaveBeenCalledWith('Extension not found: ext4');
       consoleSpy.mockRestore();
     });
@@ -536,8 +609,10 @@ describe('extension tests', () => {
       it('should be false if autoUpdate is not set in install metadata', () => {
         const activeExtensions = annotateActiveExtensions(
           extensions,
-          [],
           tempHomeDir,
+          new ExtensionEnablementManager(
+            ExtensionStorage.getUserExtensionsDir(),
+          ),
         );
         expect(
           activeExtensions.every(
@@ -556,8 +631,10 @@ describe('extension tests', () => {
         }));
         const activeExtensions = annotateActiveExtensions(
           extensionsWithAutoUpdate,
-          [],
           tempHomeDir,
+          new ExtensionEnablementManager(
+            ExtensionStorage.getUserExtensionsDir(),
+          ),
         );
         expect(
           activeExtensions.every((e) => e.installMetadata?.autoUpdate === true),
@@ -594,8 +671,10 @@ describe('extension tests', () => {
         ];
         const activeExtensions = annotateActiveExtensions(
           extensionsWithAutoUpdate,
-          [],
           tempHomeDir,
+          new ExtensionEnablementManager(
+            ExtensionStorage.getUserExtensionsDir(),
+          ),
         );
         expect(
           activeExtensions.find((e) => e.name === 'ext1')?.installMetadata
@@ -623,7 +702,10 @@ describe('extension tests', () => {
       const targetExtDir = path.join(userExtensionsDir, 'my-local-extension');
       const metadataPath = path.join(targetExtDir, INSTALL_METADATA_FILENAME);
 
-      await installExtension({ source: sourceExtDir, type: 'local' });
+      await installExtension(
+        { source: sourceExtDir, type: 'local' },
+        async (_) => true,
+      );
 
       expect(fs.existsSync(targetExtDir)).toBe(true);
       expect(fs.existsSync(metadataPath)).toBe(true);
@@ -641,9 +723,15 @@ describe('extension tests', () => {
         name: 'my-local-extension',
         version: '1.0.0',
       });
-      await installExtension({ source: sourceExtDir, type: 'local' });
+      await installExtension(
+        { source: sourceExtDir, type: 'local' },
+        async (_) => true,
+      );
       await expect(
-        installExtension({ source: sourceExtDir, type: 'local' }),
+        installExtension(
+          { source: sourceExtDir, type: 'local' },
+          async (_) => true,
+        ),
       ).rejects.toThrow(
         'Extension "my-local-extension" is already installed. Please uninstall it first.',
       );
@@ -655,7 +743,10 @@ describe('extension tests', () => {
       const configPath = path.join(sourceExtDir, EXTENSIONS_CONFIG_FILENAME);
 
       await expect(
-        installExtension({ source: sourceExtDir, type: 'local' }),
+        installExtension(
+          { source: sourceExtDir, type: 'local' },
+          async (_) => true,
+        ),
       ).rejects.toThrow(`Configuration file not found at ${configPath}`);
 
       const targetExtDir = path.join(userExtensionsDir, 'bad-extension');
@@ -669,7 +760,10 @@ describe('extension tests', () => {
       fs.writeFileSync(configPath, '{ "name": "bad-json", "version": "1.0.0"'); // Malformed JSON
 
       await expect(
-        installExtension({ source: sourceExtDir, type: 'local' }),
+        installExtension(
+          { source: sourceExtDir, type: 'local' },
+          async (_) => true,
+        ),
       ).rejects.toThrow(
         new RegExp(
           `^Failed to load extension config from ${configPath.replace(
@@ -691,7 +785,10 @@ describe('extension tests', () => {
       fs.writeFileSync(configPath, JSON.stringify({ version: '1.0.0' }));
 
       await expect(
-        installExtension({ source: sourceExtDir, type: 'local' }),
+        installExtension(
+          { source: sourceExtDir, type: 'local' },
+          async (_) => true,
+        ),
       ).rejects.toThrow(
         `Invalid configuration in ${configPath}: missing "name"`,
       );
@@ -714,7 +811,10 @@ describe('extension tests', () => {
       });
       mockGit.getRemotes.mockResolvedValue([{ name: 'origin' }]);
 
-      await installExtension({ source: gitUrl, type: 'git' });
+      await installExtension(
+        { source: gitUrl, type: 'git' },
+        async (_) => true,
+      );
 
       expect(fs.existsSync(targetExtDir)).toBe(true);
       expect(fs.existsSync(metadataPath)).toBe(true);
@@ -736,7 +836,10 @@ describe('extension tests', () => {
       const metadataPath = path.join(targetExtDir, INSTALL_METADATA_FILENAME);
       const configPath = path.join(targetExtDir, EXTENSIONS_CONFIG_FILENAME);
 
-      await installExtension({ source: sourceExtDir, type: 'link' });
+      await installExtension(
+        { source: sourceExtDir, type: 'link' },
+        async (_) => true,
+      );
 
       expect(fs.existsSync(targetExtDir)).toBe(true);
       expect(fs.existsSync(metadataPath)).toBe(true);
@@ -758,7 +861,10 @@ describe('extension tests', () => {
         version: '1.0.0',
       });
 
-      await installExtension({ source: sourceExtDir, type: 'local' });
+      await installExtension(
+        { source: sourceExtDir, type: 'local' },
+        async (_) => true,
+      );
 
       expect(mockLogExtensionInstallEvent).toHaveBeenCalled();
     });
@@ -785,12 +891,15 @@ describe('extension tests', () => {
       mockQuestion.mockImplementation((_query, callback) => callback('y'));
 
       await expect(
-        installExtension({ source: sourceExtDir, type: 'local' }, true),
+        installExtension(
+          { source: sourceExtDir, type: 'local' },
+          requestConsentNonInteractive,
+        ),
       ).resolves.toBe('my-local-extension');
 
       expect(consoleInfoSpy).toHaveBeenCalledWith(
-        `Extensions may introduce unexpected behavior.
-Ensure you have investigated the extension source and trust the author.
+        `Installing extension "my-local-extension".
+**Extensions may introduce unexpected behavior. Ensure you have investigated the extension source and trust the author.**
 This extension will run the following MCP servers:
   * test-server (local): node server.js
   * test-server-2 (remote): https://google.com`,
@@ -813,7 +922,10 @@ This extension will run the following MCP servers:
       mockQuestion.mockImplementation((_query, callback) => callback('y'));
 
       await expect(
-        installExtension({ source: sourceExtDir, type: 'local' }, true),
+        installExtension(
+          { source: sourceExtDir, type: 'local' },
+          requestConsentNonInteractive,
+        ),
       ).resolves.toBe('my-local-extension');
 
       expect(mockQuestion).toHaveBeenCalledWith(
@@ -838,8 +950,11 @@ This extension will run the following MCP servers:
       mockQuestion.mockImplementation((_query, callback) => callback('n'));
 
       await expect(
-        installExtension({ source: sourceExtDir, type: 'local' }, true),
-      ).rejects.toThrow('Installation cancelled by user.');
+        installExtension(
+          { source: sourceExtDir, type: 'local' },
+          requestConsentNonInteractive,
+        ),
+      ).rejects.toThrow('Installation cancelled for "my-local-extension".');
 
       expect(mockQuestion).toHaveBeenCalledWith(
         expect.stringContaining('Do you want to continue? [Y/n]: '),
@@ -856,11 +971,14 @@ This extension will run the following MCP servers:
       const targetExtDir = path.join(userExtensionsDir, 'my-local-extension');
       const metadataPath = path.join(targetExtDir, INSTALL_METADATA_FILENAME);
 
-      await installExtension({
-        source: sourceExtDir,
-        type: 'local',
-        autoUpdate: true,
-      });
+      await installExtension(
+        {
+          source: sourceExtDir,
+          type: 'local',
+          autoUpdate: true,
+        },
+        async (_) => true,
+      );
 
       expect(fs.existsSync(targetExtDir)).toBe(true);
       expect(fs.existsSync(metadataPath)).toBe(true);
@@ -886,9 +1004,34 @@ This extension will run the following MCP servers:
         },
       });
 
+      const mockRequestConsent = vi.fn();
+
       await expect(
-        installExtension({ source: sourceExtDir, type: 'local' }, false),
+        installExtension(
+          { source: sourceExtDir, type: 'local' },
+          mockRequestConsent,
+          process.cwd(),
+          // Provide its own existing config as the previous config.
+          await loadExtensionConfig({
+            extensionDir: sourceExtDir,
+            workspaceDir: process.cwd(),
+          }),
+        ),
       ).resolves.toBe('my-local-extension');
+
+      expect(mockRequestConsent).not.toHaveBeenCalled();
+    });
+
+    it('should throw an error for invalid extension names', async () => {
+      const sourceExtDir = createExtension({
+        extensionsDir: tempHomeDir,
+        name: 'bad_name',
+        version: '1.0.0',
+      });
+
+      await expect(
+        installExtension({ source: sourceExtDir, type: 'local' }),
+      ).rejects.toThrow('Invalid extension name: "bad_name"');
     });
   });
 
@@ -920,7 +1063,13 @@ This extension will run the following MCP servers:
       await uninstallExtension('my-local-extension');
 
       expect(fs.existsSync(sourceExtDir)).toBe(false);
-      expect(loadExtensions()).toHaveLength(1);
+      expect(
+        loadExtensions(
+          new ExtensionEnablementManager(
+            ExtensionStorage.getUserExtensionsDir(),
+          ),
+        ),
+      ).toHaveLength(1);
       expect(fs.existsSync(otherExtDir)).toBe(true);
     });
 
@@ -1024,12 +1173,15 @@ This extension will run the following MCP servers:
           version: '1.0.0',
         });
 
-        await performWorkspaceExtensionMigration([
-          loadExtension({
-            extensionDir: ext1Path,
-            workspaceDir: tempWorkspaceDir,
-          })!,
-        ]);
+        await performWorkspaceExtensionMigration(
+          [
+            loadExtension({
+              extensionDir: ext1Path,
+              workspaceDir: tempWorkspaceDir,
+            })!,
+          ],
+          async (_) => true,
+        );
 
         const userExtensionsDir = path.join(
           tempHomeDir,
@@ -1047,13 +1199,20 @@ This extension will run the following MCP servers:
           version: '1.0.0',
         });
 
-        await performWorkspaceExtensionMigration([
-          loadExtension({
-            extensionDir: ext1Path,
-            workspaceDir: tempWorkspaceDir,
-          })!,
-        ]);
-        const extensions = loadExtensions();
+        await performWorkspaceExtensionMigration(
+          [
+            loadExtension({
+              extensionDir: ext1Path,
+              workspaceDir: tempWorkspaceDir,
+            })!,
+          ],
+          async (_) => true,
+        );
+        const extensions = loadExtensions(
+          new ExtensionEnablementManager(
+            ExtensionStorage.getUserExtensionsDir(),
+          ),
+        );
 
         expect(extensions).toEqual([]);
       });
@@ -1080,8 +1239,10 @@ This extension will run the following MCP servers:
           workspaceDir: tempWorkspaceDir,
         })!,
       ];
-      const failed =
-        await performWorkspaceExtensionMigration(extensionsToMigrate);
+      const failed = await performWorkspaceExtensionMigration(
+        extensionsToMigrate,
+        async (_) => true,
+      );
 
       expect(failed).toEqual([]);
 
@@ -1091,7 +1252,9 @@ This extension will run the following MCP servers:
         'extensions',
       );
       const userExt1Path = path.join(userExtensionsDir, 'ext1');
-      const extensions = loadExtensions();
+      const extensions = loadExtensions(
+        new ExtensionEnablementManager(ExtensionStorage.getUserExtensionsDir()),
+      );
 
       expect(extensions).toHaveLength(2);
       const metadataPath = path.join(userExt1Path, INSTALL_METADATA_FILENAME);
@@ -1122,13 +1285,22 @@ This extension will run the following MCP servers:
         },
       ];
 
-      const failed = await performWorkspaceExtensionMigration(extensions);
+      const failed = await performWorkspaceExtensionMigration(
+        extensions,
+        async (_) => true,
+      );
       expect(failed).toEqual(['ext2']);
     });
   });
 
   describe('disableExtension', () => {
     it('should disable an extension at the user scope', () => {
+      createExtension({
+        extensionsDir: userExtensionsDir,
+        name: 'my-extension',
+        version: '1.0.0',
+      });
+
       disableExtension('my-extension', SettingScope.User);
       expect(
         isEnabled({
@@ -1140,6 +1312,12 @@ This extension will run the following MCP servers:
     });
 
     it('should disable an extension at the workspace scope', () => {
+      createExtension({
+        extensionsDir: userExtensionsDir,
+        name: 'my-extension',
+        version: '1.0.0',
+      });
+
       disableExtension(
         'my-extension',
         SettingScope.Workspace,
@@ -1162,6 +1340,12 @@ This extension will run the following MCP servers:
     });
 
     it('should handle disabling the same extension twice', () => {
+      createExtension({
+        extensionsDir: userExtensionsDir,
+        name: 'my-extension',
+        version: '1.0.0',
+      });
+
       disableExtension('my-extension', SettingScope.User);
       disableExtension('my-extension', SettingScope.User);
       expect(
@@ -1178,6 +1362,22 @@ This extension will run the following MCP servers:
         disableExtension('my-extension', SettingScope.System),
       ).toThrow('System and SystemDefaults scopes are not supported.');
     });
+
+    it('should log a disable event', () => {
+      createExtension({
+        extensionsDir: userExtensionsDir,
+        name: 'ext1',
+        version: '1.0.0',
+      });
+
+      disableExtension('ext1', SettingScope.Workspace);
+
+      expect(mockLogExtensionDisable).toHaveBeenCalled();
+      expect(ExtensionDisableEvent).toHaveBeenCalledWith(
+        'ext1',
+        SettingScope.Workspace,
+      );
+    });
   });
 
   describe('enableExtension', () => {
@@ -1186,11 +1386,14 @@ This extension will run the following MCP servers:
     });
 
     const getActiveExtensions = (): GeminiCLIExtension[] => {
-      const extensions = loadExtensions();
+      const manager = new ExtensionEnablementManager(
+        ExtensionStorage.getUserExtensionsDir(),
+      );
+      const extensions = loadExtensions(manager);
       const activeExtensions = annotateActiveExtensions(
         extensions,
-        [],
         tempWorkspaceDir,
+        manager,
       );
       return activeExtensions.filter((e) => e.isActive);
     };
