@@ -20,7 +20,7 @@ export const docsCommand: SlashCommand = {
   action: async (context: CommandContext): Promise<void> => {
     const docsUrl = 'https://goo.gle/gemini-cli-docs';
 
-    if (process.env.SANDBOX && process.env.SANDBOX !== 'sandbox-exec') {
+    if (process.env['SANDBOX'] && process.env['SANDBOX'] !== 'sandbox-exec') {
       context.ui.addItem(
         {
           type: MessageType.INFO,
