@@ -8,7 +8,7 @@ import { render } from 'ink-testing-library';
 import { describe, it, expect, vi } from 'vitest';
 import { StatsDisplay } from './StatsDisplay.js';
 import * as SessionContext from '../contexts/SessionContext.js';
-import { SessionMetrics } from '../contexts/SessionContext.js';
+import type { SessionMetrics } from '../contexts/SessionContext.js';
 
 // Mock the context to provide controlled data for testing
 vi.mock('../contexts/SessionContext.js', async (importOriginal) => {
@@ -24,6 +24,7 @@ const useSessionStatsMock = vi.mocked(SessionContext.useSessionStats);
 const renderWithMockedStats = (metrics: SessionMetrics) => {
   useSessionStatsMock.mockReturnValue({
     stats: {
+      sessionId: 'test-session-id',
       sessionStartTime: new Date(),
       metrics,
       lastPromptTokenCount: 0,
@@ -49,13 +50,17 @@ describe('<StatsDisplay />', () => {
         totalDecisions: { accept: 0, reject: 0, modify: 0 },
         byName: {},
       },
+      files: {
+        totalLinesAdded: 0,
+        totalLinesRemoved: 0,
+      },
     };
 
     const { lastFrame } = renderWithMockedStats(zeroMetrics);
     const output = lastFrame();
 
     expect(output).toContain('Performance');
-    expect(output).not.toContain('Interaction Summary');
+    expect(output).toContain('Interaction Summary');
     expect(output).not.toContain('Efficiency & Optimizations');
     expect(output).not.toContain('Model'); // The table header
     expect(output).toMatchSnapshot();
@@ -95,6 +100,10 @@ describe('<StatsDisplay />', () => {
         totalDecisions: { accept: 0, reject: 0, modify: 0 },
         byName: {},
       },
+      files: {
+        totalLinesAdded: 0,
+        totalLinesRemoved: 0,
+      },
     };
 
     const { lastFrame } = renderWithMockedStats(metrics);
@@ -138,6 +147,10 @@ describe('<StatsDisplay />', () => {
           },
         },
       },
+      files: {
+        totalLinesAdded: 0,
+        totalLinesRemoved: 0,
+      },
     };
 
     const { lastFrame } = renderWithMockedStats(metrics);
@@ -171,6 +184,10 @@ describe('<StatsDisplay />', () => {
             },
           },
         },
+        files: {
+          totalLinesAdded: 0,
+          totalLinesRemoved: 0,
+        },
       };
 
       const { lastFrame } = renderWithMockedStats(metrics);
@@ -205,6 +222,10 @@ describe('<StatsDisplay />', () => {
           totalDecisions: { accept: 0, reject: 0, modify: 0 },
           byName: {},
         },
+        files: {
+          totalLinesAdded: 0,
+          totalLinesRemoved: 0,
+        },
       };
 
       const { lastFrame } = renderWithMockedStats(metrics);
@@ -227,6 +248,10 @@ describe('<StatsDisplay />', () => {
           totalDecisions: { accept: 0, reject: 0, modify: 0 },
           byName: {},
         },
+        files: {
+          totalLinesAdded: 0,
+          totalLinesRemoved: 0,
+        },
       };
       const { lastFrame } = renderWithMockedStats(metrics);
       expect(lastFrame()).toMatchSnapshot();
@@ -243,6 +268,10 @@ describe('<StatsDisplay />', () => {
           totalDecisions: { accept: 0, reject: 0, modify: 0 },
           byName: {},
         },
+        files: {
+          totalLinesAdded: 0,
+          totalLinesRemoved: 0,
+        },
       };
       const { lastFrame } = renderWithMockedStats(metrics);
       expect(lastFrame()).toMatchSnapshot();
@@ -259,12 +288,68 @@ describe('<StatsDisplay />', () => {
           totalDecisions: { accept: 0, reject: 0, modify: 0 },
           byName: {},
         },
+        files: {
+          totalLinesAdded: 0,
+          totalLinesRemoved: 0,
+        },
       };
       const { lastFrame } = renderWithMockedStats(metrics);
       expect(lastFrame()).toMatchSnapshot();
     });
   });
 
+  describe('Code Changes Display', () => {
+    it('displays Code Changes when line counts are present', () => {
+      const metrics: SessionMetrics = {
+        models: {},
+        tools: {
+          totalCalls: 1,
+          totalSuccess: 1,
+          totalFail: 0,
+          totalDurationMs: 100,
+          totalDecisions: { accept: 0, reject: 0, modify: 0 },
+          byName: {},
+        },
+        files: {
+          totalLinesAdded: 42,
+          totalLinesRemoved: 18,
+        },
+      };
+
+      const { lastFrame } = renderWithMockedStats(metrics);
+      const output = lastFrame();
+
+      expect(output).toContain('Code Changes:');
+      expect(output).toContain('+42');
+      expect(output).toContain('-18');
+      expect(output).toMatchSnapshot();
+    });
+
+    it('hides Code Changes when no lines are added or removed', () => {
+      const metrics: SessionMetrics = {
+        models: {},
+        tools: {
+          totalCalls: 1,
+          totalSuccess: 1,
+          totalFail: 0,
+          totalDurationMs: 100,
+          totalDecisions: { accept: 0, reject: 0, modify: 0 },
+          byName: {},
+        },
+        files: {
+          totalLinesAdded: 0,
+          totalLinesRemoved: 0,
+        },
+      };
+
+      const { lastFrame } = renderWithMockedStats(metrics);
+      const output = lastFrame();
+
+      expect(output).not.toContain('Code Changes:');
+      expect(output).toMatchSnapshot();
+    });
+  });
+
   describe('Title Rendering', () => {
     const zeroMetrics: SessionMetrics = {
       models: {},
@@ -276,6 +361,10 @@ describe('<StatsDisplay />', () => {
         totalDecisions: { accept: 0, reject: 0, modify: 0 },
         byName: {},
       },
+      files: {
+        totalLinesAdded: 0,
+        totalLinesRemoved: 0,
+      },
     };
 
     it('renders the default title when no title prop is provided', () => {
@@ -289,6 +378,7 @@ describe('<StatsDisplay />', () => {
     it('renders the custom title when a title prop is provided', () => {
       useSessionStatsMock.mockReturnValue({
         stats: {
+          sessionId: 'test-session-id',
           sessionStartTime: new Date(),
           metrics: zeroMetrics,
           lastPromptTokenCount: 0,
