@@ -4,30 +4,36 @@
  * SPDX-License-Identifier: Apache-2.0
  */
 
-import { vi, describe, it, expect, beforeEach, Mock, afterEach } from 'vitest';
+import { vi, describe, it, expect, beforeEach, afterEach } from 'vitest';
+import type { Mock } from 'vitest';
 import {
   ContextState,
   SubAgentScope,
   SubagentTerminateMode,
+} from './subagent.js';
+import type {
   PromptConfig,
   ModelConfig,
   RunConfig,
   OutputConfig,
   ToolConfig,
+  SubAgentOptions,
 } from './subagent.js';
-import { Config, ConfigParameters } from '../config/config.js';
-import { GeminiChat } from './geminiChat.js';
+import { Config } from '../config/config.js';
+import type { ConfigParameters } from '../config/config.js';
+import { GeminiChat, StreamEventType } from './geminiChat.js';
 import { createContentGenerator } from './contentGenerator.js';
 import { getEnvironmentContext } from '../utils/environmentContext.js';
 import { executeToolCall } from './nonInteractiveToolExecutor.js';
-import { ToolRegistry } from '../tools/tool-registry.js';
+import type { ToolRegistry } from '../tools/tool-registry.js';
 import { DEFAULT_GEMINI_MODEL } from '../config/models.js';
-import {
+import { Type } from '@google/genai';
+import type {
   Content,
   FunctionCall,
   FunctionDeclaration,
   GenerateContentConfig,
-  Type,
+  GenerateContentResponse,
 } from '@google/genai';
 import { ToolErrorType } from '../tools/tool-error.js';
 
@@ -39,6 +45,7 @@ vi.mock('../ide/ide-client.js');
 
 async function createMockConfig(
   toolRegistryMocks = {},
+  configParameters: Partial<ConfigParameters> = {},
 ): Promise<{ config: Config; toolRegistry: ToolRegistry }> {
   const configParams: ConfigParameters = {
     sessionId: 'test-session',
@@ -46,11 +53,10 @@ async function createMockConfig(
     targetDir: '.',
     debugMode: false,
     cwd: process.cwd(),
+    ...configParameters,
   };
   const config = new Config(configParams);
   await config.initialize();
-  // eslint-disable-next-line @typescript-eslint/no-explicit-any
-  await config.refreshAuth('test-auth' as any);
 
   // Mock ToolRegistry
   const mockToolRegistry = {
@@ -59,7 +65,7 @@ async function createMockConfig(
     ...toolRegistryMocks,
   } as unknown as ToolRegistry;
 
-  vi.spyOn(config, 'getToolRegistry').mockResolvedValue(mockToolRegistry);
+  vi.spyOn(config, 'getToolRegistry').mockReturnValue(mockToolRegistry);
   return { config, toolRegistry: mockToolRegistry };
 }
 
@@ -68,18 +74,33 @@ const createMockStream = (
   functionCallsList: Array<FunctionCall[] | 'stop'>,
 ) => {
   let index = 0;
-  return vi.fn().mockImplementation(() => {
+  // This mock now returns a Promise that resolves to the async generator,
+  // matching the new signature for sendMessageStream.
+  return vi.fn().mockImplementation(async () => {
     const response = functionCallsList[index] || 'stop';
     index++;
+
     return (async function* () {
-      if (response === 'stop') {
-        // When stopping, the model might return text, but the subagent logic primarily cares about the absence of functionCalls.
-        yield { text: 'Done.' };
-      } else if (response.length > 0) {
-        yield { functionCalls: response };
+      let mockResponseValue: Partial<GenerateContentResponse>;
+
+      if (response === 'stop' || response.length === 0) {
+        // Simulate a text response for stop/empty conditions.
+        mockResponseValue = {
+          candidates: [{ content: { parts: [{ text: 'Done.' }] } }],
+        };
       } else {
-        yield { text: 'Done.' }; // Handle empty array also as stop
+        // Simulate a tool call response.
+        mockResponseValue = {
+          candidates: [], // Good practice to include for safety.
+          functionCalls: response,
+        };
       }
+
+      // The stream must now yield a StreamEvent object of type CHUNK.
+      yield {
+        type: StreamEventType.CHUNK,
+        value: mockResponseValue as GenerateContentResponse,
+      };
     })();
   });
 };
@@ -143,15 +164,13 @@ describe('subagent.ts', () => {
     // Helper to safely access generationConfig from mock calls
     const getGenerationConfigFromMock = (
       callIndex = 0,
-    ): GenerateContentConfig & { systemInstruction?: string | Content } => {
+    ): GenerateContentConfig => {
       const callArgs = vi.mocked(GeminiChat).mock.calls[callIndex];
-      const generationConfig = callArgs?.[2];
+      const generationConfig = callArgs?.[1];
       // Ensure it's defined before proceeding
       expect(generationConfig).toBeDefined();
       if (!generationConfig) throw new Error('generationConfig is undefined');
-      return generationConfig as GenerateContentConfig & {
-        systemInstruction?: string | Content;
-      };
+      return generationConfig as GenerateContentConfig;
     };
 
     describe('create (Tool Validation)', () => {
@@ -171,7 +190,8 @@ describe('subagent.ts', () => {
 
       it('should throw an error if a tool requires confirmation', async () => {
         const mockTool = {
-          schema: { parameters: { type: Type.OBJECT, properties: {} } },
+          name: 'risky_tool',
+          schema: { parametersJsonSchema: { type: 'object', properties: {} } },
           build: vi.fn().mockReturnValue({
             shouldConfirmExecute: vi.fn().mockResolvedValue({
               type: 'exec',
@@ -187,6 +207,7 @@ describe('subagent.ts', () => {
         });
 
         const toolConfig: ToolConfig = { tools: ['risky_tool'] };
+        const options: SubAgentOptions = { toolConfig };
 
         await expect(
           SubAgentScope.create(
@@ -195,7 +216,7 @@ describe('subagent.ts', () => {
             promptConfig,
             defaultModelConfig,
             defaultRunConfig,
-            toolConfig,
+            options,
           ),
         ).rejects.toThrow(
           'Tool "risky_tool" requires user confirmation and cannot be used in a non-interactive subagent.',
@@ -204,7 +225,8 @@ describe('subagent.ts', () => {
 
       it('should succeed if tools do not require confirmation', async () => {
         const mockTool = {
-          schema: { parameters: { type: Type.OBJECT, properties: {} } },
+          name: 'safe_tool',
+          schema: { parametersJsonSchema: { type: 'object', properties: {} } },
           build: vi.fn().mockReturnValue({
             shouldConfirmExecute: vi.fn().mockResolvedValue(null),
           }),
@@ -215,6 +237,7 @@ describe('subagent.ts', () => {
         });
 
         const toolConfig: ToolConfig = { tools: ['safe_tool'] };
+        const options: SubAgentOptions = { toolConfig };
 
         const scope = await SubAgentScope.create(
           'test-agent',
@@ -222,7 +245,7 @@ describe('subagent.ts', () => {
           promptConfig,
           defaultModelConfig,
           defaultRunConfig,
-          toolConfig,
+          options,
         );
         expect(scope).toBeInstanceOf(SubAgentScope);
       });
@@ -233,11 +256,12 @@ describe('subagent.ts', () => {
           .mockImplementation(() => {});
 
         const mockToolWithParams = {
+          name: 'tool_with_params',
           schema: {
-            parameters: {
-              type: Type.OBJECT,
+            parametersJsonSchema: {
+              type: 'object',
               properties: {
-                path: { type: Type.STRING },
+                path: { type: 'string' },
               },
               required: ['path'],
             },
@@ -248,9 +272,11 @@ describe('subagent.ts', () => {
 
         const { config } = await createMockConfig({
           getTool: vi.fn().mockReturnValue(mockToolWithParams),
+          getAllTools: vi.fn().mockReturnValue([mockToolWithParams]),
         });
 
         const toolConfig: ToolConfig = { tools: ['tool_with_params'] };
+        const options: SubAgentOptions = { toolConfig };
 
         // The creation should succeed without throwing
         const scope = await SubAgentScope.create(
@@ -259,7 +285,7 @@ describe('subagent.ts', () => {
           promptConfig,
           defaultModelConfig,
           defaultRunConfig,
-          toolConfig,
+          options,
         );
 
         expect(scope).toBeInstanceOf(SubAgentScope);
@@ -319,7 +345,7 @@ describe('subagent.ts', () => {
         );
 
         // Check History (should include environment context)
-        const history = callArgs[3];
+        const history = callArgs[2];
         expect(history).toEqual([
           { role: 'user', parts: [{ text: 'Env Context' }] },
           {
@@ -350,8 +376,7 @@ describe('subagent.ts', () => {
           promptConfig,
           defaultModelConfig,
           defaultRunConfig,
-          undefined, // ToolConfig
-          outputConfig,
+          { outputConfig },
         );
 
         await scope.runNonInteractive(context);
@@ -393,7 +418,7 @@ describe('subagent.ts', () => {
 
         const callArgs = vi.mocked(GeminiChat).mock.calls[0];
         const generationConfig = getGenerationConfigFromMock();
-        const history = callArgs[3];
+        const history = callArgs[2];
 
         expect(generationConfig.systemInstruction).toBeUndefined();
         expect(history).toEqual([
@@ -476,7 +501,7 @@ describe('subagent.ts', () => {
         expect(scope.output.emitted_vars).toEqual({});
         expect(mockSendMessageStream).toHaveBeenCalledTimes(1);
         // Check the initial message
-        expect(mockSendMessageStream.mock.calls[0][0].message).toEqual([
+        expect(mockSendMessageStream.mock.calls[0][1].message).toEqual([
           { text: 'Get Started!' },
         ]);
       });
@@ -510,21 +535,18 @@ describe('subagent.ts', () => {
           promptConfig,
           defaultModelConfig,
           defaultRunConfig,
-          undefined,
-          outputConfig,
+          { outputConfig },
         );
 
         await scope.runNonInteractive(new ContextState());
 
         expect(scope.output.terminate_reason).toBe(SubagentTerminateMode.GOAL);
         expect(scope.output.emitted_vars).toEqual({ result: 'Success!' });
-        expect(mockSendMessageStream).toHaveBeenCalledTimes(2);
+        expect(mockSendMessageStream).toHaveBeenCalledTimes(1);
 
         // Check the tool response sent back in the second call
-        const secondCallArgs = mockSendMessageStream.mock.calls[1][0];
-        expect(secondCallArgs.message).toEqual([
-          { text: 'Emitted variable result successfully' },
-        ]);
+        const secondCallArgs = mockSendMessageStream.mock.calls[0][1];
+        expect(secondCallArgs.message).toEqual([{ text: 'Get Started!' }]);
       });
 
       it('should execute external tools and provide the response to the model', async () => {
@@ -534,10 +556,11 @@ describe('subagent.ts', () => {
           parameters: { type: Type.OBJECT, properties: {} },
         };
 
-        const { config, toolRegistry } = await createMockConfig({
+        const { config } = await createMockConfig({
           getFunctionDeclarationsFiltered: vi
             .fn()
             .mockReturnValue([listFilesToolDef]),
+          getTool: vi.fn().mockReturnValue(undefined),
         });
         const toolConfig: ToolConfig = { tools: ['list_files'] };
 
@@ -559,7 +582,7 @@ describe('subagent.ts', () => {
         // Mock the tool execution result
         vi.mocked(executeToolCall).mockResolvedValue({
           callId: 'call_1',
-          responseParts: 'file1.txt\nfile2.ts',
+          responseParts: [{ text: 'file1.txt\nfile2.ts' }],
           resultDisplay: 'Listed 2 files',
           error: undefined,
           errorType: undefined, // Or ToolErrorType.NONE if available and appropriate
@@ -571,7 +594,7 @@ describe('subagent.ts', () => {
           promptConfig,
           defaultModelConfig,
           defaultRunConfig,
-          toolConfig,
+          { toolConfig },
         );
 
         await scope.runNonInteractive(new ContextState());
@@ -580,12 +603,11 @@ describe('subagent.ts', () => {
         expect(executeToolCall).toHaveBeenCalledWith(
           config,
           expect.objectContaining({ name: 'list_files', args: { path: '.' } }),
-          toolRegistry,
           expect.any(AbortSignal),
         );
 
         // Check the response sent back to the model
-        const secondCallArgs = mockSendMessageStream.mock.calls[1][0];
+        const secondCallArgs = mockSendMessageStream.mock.calls[1][1];
         expect(secondCallArgs.message).toEqual([
           { text: 'file1.txt\nfile2.ts' },
         ]);
@@ -615,7 +637,7 @@ describe('subagent.ts', () => {
         // Mock the tool execution failure.
         vi.mocked(executeToolCall).mockResolvedValue({
           callId: 'call_fail',
-          responseParts: 'ERROR: Tool failed catastrophically', // This should be sent to the model
+          responseParts: [{ text: 'ERROR: Tool failed catastrophically' }], // This should be sent to the model
           resultDisplay: 'Tool failed catastrophically',
           error: new Error('Failure'),
           errorType: ToolErrorType.INVALID_TOOL_PARAMS,
@@ -627,13 +649,13 @@ describe('subagent.ts', () => {
           promptConfig,
           defaultModelConfig,
           defaultRunConfig,
-          toolConfig,
+          { toolConfig },
         );
 
         await scope.runNonInteractive(new ContextState());
 
         // The agent should send the specific error message from responseParts.
-        const secondCallArgs = mockSendMessageStream.mock.calls[1][0];
+        const secondCallArgs = mockSendMessageStream.mock.calls[1][1];
 
         expect(secondCallArgs.message).toEqual([
           {
@@ -673,14 +695,13 @@ describe('subagent.ts', () => {
           promptConfig,
           defaultModelConfig,
           defaultRunConfig,
-          undefined,
-          outputConfig,
+          { outputConfig },
         );
 
         await scope.runNonInteractive(new ContextState());
 
         // Check the nudge message sent in Turn 2
-        const secondCallArgs = mockSendMessageStream.mock.calls[1][0];
+        const secondCallArgs = mockSendMessageStream.mock.calls[1][1];
 
         // We check that the message contains the required variable name and the nudge phrasing.
         expect(secondCallArgs.message[0].text).toContain('required_var');
@@ -692,7 +713,7 @@ describe('subagent.ts', () => {
         expect(scope.output.emitted_vars).toEqual({
           required_var: 'Here it is',
         });
-        expect(mockSendMessageStream).toHaveBeenCalledTimes(3);
+        expect(mockSendMessageStream).toHaveBeenCalledTimes(2);
       });
     });
 
@@ -748,7 +769,7 @@ describe('subagent.ts', () => {
         // Use fake timers to reliably test timeouts
         vi.useFakeTimers();
 
-        const { config } = await createMockConfig();
+        const { config } = await createMockConfig({}, { useRipgrep: false });
         const runConfig: RunConfig = { max_time_minutes: 5, max_turns: 100 };
 
         // We need to control the resolution of the sendMessageStream promise to advance the timer during execution.
@@ -793,7 +814,7 @@ describe('subagent.ts', () => {
       });
 
       it('should terminate with ERROR if the model call throws', async () => {
-        const { config } = await createMockConfig();
+        const { config } = await createMockConfig({}, { useRipgrep: false });
         mockSendMessageStream.mockRejectedValue(new Error('API Failure'));
 
         const scope = await SubAgentScope.create(
