@@ -32,6 +32,8 @@ import {
   EVENT_RIPGREP_FALLBACK,
   EVENT_MODEL_ROUTING,
   EVENT_EXTENSION_INSTALL,
+  EVENT_MODEL_SLASH_COMMAND,
+  EVENT_EXTENSION_DISABLE,
 } from './constants.js';
 import type {
   ApiErrorEvent,
@@ -57,9 +59,11 @@ import type {
   RipgrepFallbackEvent,
   ToolOutputTruncatedEvent,
   ModelRoutingEvent,
+  ExtensionDisableEvent,
   ExtensionEnableEvent,
   ExtensionUninstallEvent,
   ExtensionInstallEvent,
+  ModelSlashCommandEvent,
 } from './types.js';
 import {
   recordApiErrorMetrics,
@@ -72,6 +76,7 @@ import {
   recordContentRetry,
   recordContentRetryFailure,
   recordModelRoutingMetrics,
+  recordModelSlashCommand,
 } from './metrics.js';
 import { isTelemetrySdkInitialized } from './sdk.js';
 import type { UiEvent } from './uiTelemetry.js';
@@ -186,14 +191,12 @@ export function logToolCall(config: Config, event: ToolCallEvent): void {
     attributes,
   };
   logger.emit(logRecord);
-  recordToolCallMetrics(
-    config,
-    event.function_name,
-    event.duration_ms,
-    event.success,
-    event.decision,
-    event.tool_type,
-  );
+  recordToolCallMetrics(config, event.duration_ms, {
+    function_name: event.function_name,
+    success: event.success,
+    decision: event.decision,
+    tool_type: event.tool_type,
+  });
 }
 
 export function logToolOutputTruncated(
@@ -253,14 +256,13 @@ export function logFileOperation(
   };
   logger.emit(logRecord);
 
-  recordFileOperationMetric(
-    config,
-    event.operation,
-    event.lines,
-    event.mimetype,
-    event.extension,
-    event.programming_language,
-  );
+  recordFileOperationMetric(config, {
+    operation: event.operation,
+    lines: event.lines,
+    mimetype: event.mimetype,
+    extension: event.extension,
+    programming_language: event.programming_language,
+  });
 }
 
 export function logApiRequest(config: Config, event: ApiRequestEvent): void {
@@ -359,13 +361,11 @@ export function logApiError(config: Config, event: ApiErrorEvent): void {
     attributes,
   };
   logger.emit(logRecord);
-  recordApiErrorMetrics(
-    config,
-    event.model,
-    event.duration_ms,
-    event.status_code,
-    event.error_type,
-  );
+  recordApiErrorMetrics(config, event.duration_ms, {
+    model: event.model,
+    status_code: event.status_code,
+    error_type: event.error_type,
+  });
 }
 
 export function logApiResponse(config: Config, event: ApiResponseEvent): void {
@@ -398,37 +398,30 @@ export function logApiResponse(config: Config, event: ApiResponseEvent): void {
     attributes,
   };
   logger.emit(logRecord);
-  recordApiResponseMetrics(
-    config,
-    event.model,
-    event.duration_ms,
-    event.status_code,
-  );
-  recordTokenUsageMetrics(
-    config,
-    event.model,
-    event.input_token_count,
-    'input',
-  );
-  recordTokenUsageMetrics(
-    config,
-    event.model,
-    event.output_token_count,
-    'output',
-  );
-  recordTokenUsageMetrics(
-    config,
-    event.model,
-    event.cached_content_token_count,
-    'cache',
-  );
-  recordTokenUsageMetrics(
-    config,
-    event.model,
-    event.thoughts_token_count,
-    'thought',
-  );
-  recordTokenUsageMetrics(config, event.model, event.tool_token_count, 'tool');
+  recordApiResponseMetrics(config, event.duration_ms, {
+    model: event.model,
+    status_code: event.status_code,
+  });
+  recordTokenUsageMetrics(config, event.input_token_count, {
+    model: event.model,
+    type: 'input',
+  });
+  recordTokenUsageMetrics(config, event.output_token_count, {
+    model: event.model,
+    type: 'output',
+  });
+  recordTokenUsageMetrics(config, event.cached_content_token_count, {
+    model: event.model,
+    type: 'cache',
+  });
+  recordTokenUsageMetrics(config, event.thoughts_token_count, {
+    model: event.model,
+    type: 'thought',
+  });
+  recordTokenUsageMetrics(config, event.tool_token_count, {
+    model: event.model,
+    type: 'tool',
+  });
 }
 
 export function logLoopDetected(
@@ -698,6 +691,28 @@ export function logModelRouting(
   recordModelRoutingMetrics(config, event);
 }
 
+export function logModelSlashCommand(
+  config: Config,
+  event: ModelSlashCommandEvent,
+): void {
+  ClearcutLogger.getInstance(config)?.logModelSlashCommandEvent(event);
+  if (!isTelemetrySdkInitialized()) return;
+
+  const attributes: LogAttributes = {
+    ...getCommonAttributes(config),
+    ...event,
+    'event.name': EVENT_MODEL_SLASH_COMMAND,
+  };
+
+  const logger = logs.getLogger(SERVICE_NAME);
+  const logRecord: LogRecord = {
+    body: `Model slash command. Model: ${event.model_name}`,
+    attributes,
+  };
+  logger.emit(logRecord);
+  recordModelSlashCommand(config, event);
+}
+
 export function logExtensionInstallEvent(
   config: Config,
   event: ExtensionInstallEvent,
@@ -767,3 +782,25 @@ export function logExtensionEnable(
   };
   logger.emit(logRecord);
 }
+
+export function logExtensionDisable(
+  config: Config,
+  event: ExtensionDisableEvent,
+): void {
+  ClearcutLogger.getInstance(config)?.logExtensionDisableEvent(event);
+  if (!isTelemetrySdkInitialized()) return;
+
+  const attributes: LogAttributes = {
+    ...getCommonAttributes(config),
+    ...event,
+    'event.name': EVENT_EXTENSION_DISABLE,
+    'event.timestamp': new Date().toISOString(),
+  };
+
+  const logger = logs.getLogger(SERVICE_NAME);
+  const logRecord: LogRecord = {
+    body: `Disabled extension ${event.extension_name}`,
+    attributes,
+  };
+  logger.emit(logRecord);
+}
