@@ -10,6 +10,13 @@ import type { DiffUpdateResult } from '../ide/ide-client.js';
 import type { ShellExecutionConfig } from '../services/shellExecutionService.js';
 import { SchemaValidator } from '../utils/schemaValidator.js';
 import type { AnsiOutput } from '../utils/terminalSerializer.js';
+import type { MessageBus } from '../confirmation-bus/message-bus.js';
+import { randomUUID } from 'node:crypto';
+import {
+  MessageBusType,
+  type ToolConfirmationRequest,
+  type ToolConfirmationResponse,
+} from '../confirmation-bus/types.js';
 
 /**
  * Represents a validated and ready-to-execute tool call.
@@ -66,7 +73,16 @@ export abstract class BaseToolInvocation<
   TResult extends ToolResult,
 > implements ToolInvocation<TParams, TResult>
 {
-  constructor(readonly params: TParams) {}
+  constructor(
+    readonly params: TParams,
+    protected readonly messageBus?: MessageBus,
+  ) {
+    if (this.messageBus) {
+      console.log(
+        `[DEBUG] Tool ${this.constructor.name} created with messageBus: YES`,
+      );
+    }
+  }
 
   abstract getDescription(): string;
 
@@ -77,9 +93,116 @@ export abstract class BaseToolInvocation<
   shouldConfirmExecute(
     _abortSignal: AbortSignal,
   ): Promise<ToolCallConfirmationDetails | false> {
+    // If message bus is available, use it for confirmation
+    if (this.messageBus) {
+      console.log(
+        `[DEBUG] Using message bus for tool confirmation: ${this.constructor.name}`,
+      );
+      return this.handleMessageBusConfirmation(_abortSignal);
+    }
+
+    // Fall back to existing confirmation flow
     return Promise.resolve(false);
   }
 
+  /**
+   * Handle tool confirmation using the message bus.
+   * This method publishes a confirmation request and waits for the response.
+   */
+  protected async handleMessageBusConfirmation(
+    abortSignal: AbortSignal,
+  ): Promise<ToolCallConfirmationDetails | false> {
+    if (!this.messageBus) {
+      return false;
+    }
+
+    const correlationId = randomUUID();
+    const toolCall = {
+      name: this.constructor.name,
+      args: this.params as Record<string, unknown>,
+    };
+
+    return new Promise<ToolCallConfirmationDetails | false>(
+      (resolve, reject) => {
+        if (!this.messageBus) {
+          resolve(false);
+          return;
+        }
+
+        let timeoutId: NodeJS.Timeout | undefined;
+
+        // Centralized cleanup function
+        const cleanup = () => {
+          if (timeoutId) {
+            clearTimeout(timeoutId);
+            timeoutId = undefined;
+          }
+          abortSignal.removeEventListener('abort', abortHandler);
+          this.messageBus?.unsubscribe(
+            MessageBusType.TOOL_CONFIRMATION_RESPONSE,
+            responseHandler,
+          );
+        };
+
+        // Set up abort handler
+        const abortHandler = () => {
+          cleanup();
+          reject(new Error('Tool confirmation aborted'));
+        };
+
+        // Check if already aborted
+        if (abortSignal.aborted) {
+          reject(new Error('Tool confirmation aborted'));
+          return;
+        }
+
+        // Set up response handler
+        const responseHandler = (response: ToolConfirmationResponse) => {
+          if (response.correlationId === correlationId) {
+            cleanup();
+
+            if (response.confirmed) {
+              // Tool was confirmed, return false to indicate no further confirmation needed
+              resolve(false);
+            } else {
+              // Tool was denied, reject to prevent execution
+              reject(new Error('Tool execution denied by policy'));
+            }
+          }
+        };
+
+        // Add event listener for abort signal
+        abortSignal.addEventListener('abort', abortHandler);
+
+        // Set up timeout
+        timeoutId = setTimeout(() => {
+          cleanup();
+          resolve(false);
+        }, 30000); // 30 second timeout
+
+        // Subscribe to response
+        this.messageBus.subscribe(
+          MessageBusType.TOOL_CONFIRMATION_RESPONSE,
+          responseHandler,
+        );
+
+        // Publish confirmation request
+        const request: ToolConfirmationRequest = {
+          type: MessageBusType.TOOL_CONFIRMATION_REQUEST,
+          toolCall,
+          correlationId,
+        };
+
+        try {
+          this.messageBus.publish(request);
+        } catch (_error) {
+          cleanup();
+          resolve(false);
+        }
+      },
+    );
+  }
+
   abstract execute(
     signal: AbortSignal,
     updateOutput?: (output: string | AnsiOutput) => void,
@@ -159,6 +282,7 @@ export abstract class DeclarativeTool<
     readonly parameterSchema: unknown,
     readonly isOutputMarkdown: boolean = true,
     readonly canUpdateOutput: boolean = false,
+    readonly messageBus?: MessageBus,
   ) {}
 
   get schema(): FunctionDeclaration {
@@ -282,7 +406,7 @@ export abstract class BaseDeclarativeTool<
     if (validationError) {
       throw new Error(validationError);
     }
-    return this.createInvocation(params);
+    return this.createInvocation(params, this.messageBus);
   }
 
   override validateToolParams(params: TParams): string | null {
@@ -304,6 +428,7 @@ export abstract class BaseDeclarativeTool<
 
   protected abstract createInvocation(
     params: TParams,
+    messageBus?: MessageBus,
   ): ToolInvocation<TParams, TResult>;
 }
 
