@@ -5,7 +5,10 @@
  */
 
 import * as vscode from 'vscode';
-import type { File, IdeContext } from '@google/gemini-cli-core';
+import type {
+  File,
+  IdeContext,
+} from '@google/gemini-cli-core/src/ide/types.js';
 
 export const MAX_FILES = 10;
 const MAX_SELECTED_TEXT_LENGTH = 16384; // 16 KiB limit
