@@ -46,7 +46,11 @@ for (const workspace of rootPackageJson.workspaces) {
   }
 }
 
-// Clean up vsix files in vscode-ide-companion
+// Clean up vscode-ide-companion package
+rmSync(join(root, 'packages/vscode-ide-companion/node_modules'), {
+  recursive: true,
+  force: true,
+});
 const vsixFiles = globSync('packages/vscode-ide-companion/*.vsix', {
   cwd: root,
 });
